C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      PROGRAM CORRECT_BAND26

c-----------------------------------------------------------------------
c Purpose:
c     To correct for the leaking of energy from the 1.2 micron region
c     (band 5) into the 1.38 micron detectors (Band 26).  This will fix
c     the "stripes" seen in Band 26 as well as generally lower the 
c     radiance of the 1.38 band (Band 26).  
c
c     The band 26 scaled integers will be corrected and re-written
c     back into the hdf file.  In addition, a global attritute 
c     "IMAPP_DESTRIPE" will be added to the 1 km hdf file.
c
c     The process of destriping works as such:
c        1)  Band 26 and Band 5 scaled integer data are read out of 
c            the 1 km MODIS HDF file.
c        2)  If Band 5 data is saturated (scaled integer values of
c            65533 or 65528) the largest possible good data value is
c            used for band 5.
c        3)  If the band 5 value is bad, the band 26 value is 
c            replaced as missing.
c        4)  If band 26 value is bad, it is left as bad.
c        5)  Band 5 and Band 26 data are converted to radiances.
c        6)  The correction is applied to band 26 data using this
c            formula:  Rad26cor = Rad26(i) - Rad5(i)*Coef(i)
c            where i is the instrument dectector number (1-10).
c        7)  The corrected band 26 radiances are turned back into
c            scaled integers and written back into the file.
c        8)  A global attribute is added to the file to indicate that
c            band 26 has been corrected (IMAPP_DESTRIPE).
c     
c Usage:
c     correct_band26 sat_name in_hdf coef_file
c     where
c     sat_name: platform name of 1km HDF file 
c               (Current correction  for Terra only)
c     in_hdf:  name of input MOD021KM HDF file
c     coef_file: Name of file containing b26 correction coefficients 
c     
c     Codes writes message to standard output if successful
c
c Author:
c     Kathleen Strabala kathy.strabala@ssec.wisc.edu
c-----------------------------------------------------------------------

      implicit none

c ... Include files
      include 'hdf.inc'

c ... Local variables
      integer file_id, varid_500, varid_rad, varid_B26
      integer rtn, io_err, clun, i
      integer rank, dimsizes(3), data_type, num_attrs
      character*100 sds_name
      integer num_pixels, num_lines, num_scans, num_bands
      
      integer lines_1km
      parameter (lines_1km = 10)
      
      integer num_500, num_ref, B26, B5
      parameter (num_500 = 5, num_ref = 15, B26 = 15, B5 = 3)
      
      real scale_500rad(num_500), offset_500rad(num_500)
      real scale_rad(num_ref), offset_rad(num_ref)
      integer*2 valid_range(2)
      
      integer scan
      integer start(3), stride(3), edge(3)

      integer max_pixel, max_line
      parameter (max_pixel = 1500, max_line = 10)
      
      integer max_band
      parameter (max_band = num_ref)
      
      real R500_array(max_pixel, max_line, num_500),
     &     rad_array(max_pixel, max_line, num_ref),
     &     image(max_pixel, max_line)
      integer*2 image2(max_pixel * max_line)

      integer pixel, line, index, G5, G26
      
c ... Arguments
      character*5 sat_name
      character*132 in_file, coef_file

      real bad_value
      parameter (bad_value = -1.0)
      
      integer num_arg
      
c ... Variables for band 26 correction on Terra
      integer detectors
      parameter (detectors = 10)
      real coefficients(detectors)
      
c ... External functions
      integer sfstart, sfselect, sfn2index, sfginfo, 
     &  sfendacc, sfend, sffattr, sfrnatt, sfwdata, sfscatt
      external sfstart, sfselect, sfn2index, sfginfo, 
     &  sfendacc, sfend, sffattr, sfrnatt, sfwdata, sfscatt

      integer get_modis_band_index
      external get_modis_band_index

      integer iargc
      
      integer string_length
      external string_length

c ... data statements
      data clun /40/
      
c-----------------------------------------------------------------------
c     GET INPUT ARGUMENTS
c-----------------------------------------------------------------------

c ... Check number of arguments
      num_arg = iargc()
      if (num_arg .ne. 3) then
        print *, 'Usage: correct_band26 sat_name in_hdf'
        print *, 'where'
        print *, 'sat_name: MODIS satellite platform name (Aqua or Terra)'
        print *, 'in_hdf:  name of MODIS 1km HDF file to be corrected'
        print *, 'coef_file: name of band 26 coefficient file'
        call exit(-1)
      endif

c ... Extract arguments
      call getarg(1, sat_name)
      call getarg(2, in_file)
      call getarg(3, coef_file)
          
c-----------------------------------------------------------------------
c     OPEN FILES
c-----------------------------------------------------------------------

c ... Print the input and output filenames
      print '(''Input Satellite Platform Name: '', a)', sat_name(1: string_length(sat_name))
      print '(''Input 1 km HDF file: '', a)', in_file(1: string_length(in_file))
      print '(''B26 Coefficient file: '', a)', coef_file(1: string_length(coef_file))

c ... Check input platform name to make sure it is either Aqua or Terra
      if (sat_name(1:5) .ne. 'Terra'   .and.
     +    sat_name(1:5) .ne. 'terra'   .and.
     +    sat_name(1:5) .ne. 'TERRA') then    
          print *, 'Error: Incorrect Satellite Platform name entered'
          call exit(-1)
      endif

c ... Open the input HDF file for reading and writing 
c ... (DFACC_RDWR is defined in hdf.inc)
      file_id = sfstart(in_file, DFACC_RDWR)
      if (file_id .eq. -1) then
        print *, 'Error: Could not open input MOD021KM HDF file'
        call exit(-1)
      endif

c ... Open the input correction coefficient file
      open(FILE=coef_file,UNIT=clun,STATUS='old',
     &     FORM='formatted',ACCESS='sequential',
     &     IOSTAT=io_err)
      if (io_err .ne. 0) then
        print *, 'Error: Could not open coefficient file'
        call exit(-1)
      endif

c ... Read the coefficients
      read(UNIT=clun,FMT='(10f6.3)') coefficients
      write(*,'(2x,'' Coefficients: '',10f6.3)') 
     &                (coefficients(i),i=1,10)
      
c ... Get the SDS id for each image array
      varid_rad = sfselect(file_id, sfn2index(file_id, 'EV_1KM_RefSB'))
      if (varid_rad .eq. -1) then
        print *, 'Error: Input file is not a MOD021KM HDF file'
        call exit(-1)
      endif
      varid_500 = sfselect(file_id, sfn2index(file_id, 'EV_500_Aggr1km_RefSB'))
      varid_B26 = sfselect(file_id, sfn2index(file_id, 'EV_Band26'))
      
c ... Get the number of pixels, lines, and scans
      rtn = sfginfo(varid_rad, sds_name, rank, dimsizes, data_type, num_attrs) 
      num_pixels = dimsizes(1)
      num_lines  = dimsizes(2)
      num_bands  = dimsizes(3)
      num_scans  = num_lines / lines_1km
      print '(''Pixels = '', i6)', num_pixels
      print '(''Lines  = '', i6)', num_lines
      print '(''Bands  = '', i6)', num_bands
      print '(''Scans  = '', i6)', num_scans

c ... Check to see if file has already been destriped
      if (sffattr(file_id,'IMAPP_DESTRIPE').ne. -1) then
          print *, 'Band 26 has already been destriped'
          call exit(-1)
      endif

c-----------------------------------------------------------------------
c     READ THE IMAGE DATA FROM HDF INPUT 
c-----------------------------------------------------------------------

c ... Get the scales and offsets for each image array
      rtn=sfrnatt(varid_500, sffattr(varid_500, 'radiance_scales'),scale_500rad)
      rtn=sfrnatt(varid_500, sffattr(varid_500, 'radiance_offsets'),offset_500rad)
      rtn=sfrnatt(varid_500, sffattr(varid_500, 'valid_range'),valid_range)
      rtn=sfrnatt(varid_rad, sffattr(varid_rad, 'radiance_scales'),scale_rad)
      rtn=sfrnatt(varid_rad, sffattr(varid_rad, 'radiance_offsets'),offset_rad)
      
c ... Set start vector (static values only)
      start(1) = 0

c ... Set stride vector
      stride(1) = 1
      stride(2) = 1
      stride(3) = 1
      
c ... Set edge vector (static values only)
      edge(1) = num_pixels
      edge(2) = lines_1km

c ... Loop over each scan
      do scan = 0, (num_scans - 1), 1

c ...   Report progress on every 10th scan
        if (mod(scan, 10) .eq. 0) print '(''Scan: '', i6)', scan

c ...   Set start vector
        start(2) = scan * lines_1km
        
c-----------------------------------------------------------------------
c       PERFORM THE CORRECTION
c-----------------------------------------------------------------------

c ...   Read band 5 radiance information (it is the 3rd band in
c ...    the array)
        start(3) = 0
        edge(3) = num_500
        call get_modis_l1b_valdata(varid_500, start, stride, edge,
     &     scale_500rad, offset_500rad, bad_value, valid_range,
     &     max_pixel, max_line, max_band, R500_array(1,1,1))

c ...   Read band 26 radiance information (it is the 15th band in
c ...    the array)
        start(3) = 0
        edge(3) = num_ref
        call get_modis_l1b_data(varid_rad, start, stride, edge,
     &      scale_rad, offset_rad, bad_value,
     &      max_pixel, max_line, max_band, rad_array(1,1,1))

c ...   Use correction coefficient for each detector based upon
c ...     band 5 radiance. 
        index = 1
        do line = 1, lines_1km
           do pixel = 1, num_pixels
              G5 = 1
              G26 = 1
              if (nint(R500_array(pixel,line,B5)).eq.nint(bad_value))
     &            G5 = 0
              if (nint(rad_array(pixel,line,B26)).eq.nint(bad_value))
     &            G26 = 0
              if ((G5+G26) .eq. 2)  then
                  image(pixel,line) = rad_array(pixel,line,B26) - 
     &                 (R500_array(pixel,line,B5) * coefficients(line))
c ...             Convert the data back to scaled integers
                  image2(index) = nint((image(pixel,line)/scale_rad(B26)) +
     &                                offset_rad(B26)) 
              else 
                  image2(index) = nint(bad_value)
              endif
              index = index + 1
           enddo
        enddo

c-----------------------------------------------------------------------
c       WRITE THE CORRECTED DATA BACK INTO THE HDF FILE
c-----------------------------------------------------------------------
        start(3) = B26 - 1
        edge(3) = 1
        rtn = sfwdata(varid_rad, start, stride, edge, image2)

c ...   Also need to write it into the single B26 array
        start(3) = 0
        edge(3) = 1
        rtn = sfwdata(varid_B26, start, stride, edge, image2)
           
      end do
     
c-----------------------------------------------------------------------
c     WRITE A GLOBAL ATTRIBUTE INTO THE FILE IDENTIFYING DESTRIPE
c-----------------------------------------------------------------------
c ... write Destripe attribute into HDF file
      rtn = sfscatt(file_id,'IMAPP_DESTRIPE',DFNT_CHAR8,4,'v1.1')
      if (rtn .lt. 0) 
     &  write(*,'(''Unable to attach Destrip attribute to file'')') 

      print *, 'Band 26 has been successfully destriped'
           
c-----------------------------------------------------------------------
c     CLEANUP AND EXIT
c-----------------------------------------------------------------------

c ... End access to the image arrays
      rtn = sfendacc(varid_500)
      rtn = sfendacc(varid_rad)
      rtn = sfendacc(varid_B26)

c ... Close the hdf file
      rtn = sfend(file_id)

c ... Close the coefficient file
      close(clun)

      END
